﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace AirBomber
{
    /// <summary>
    /// Класс, который генирирует самолет из разнообразного количества сущностей и типа двигателей
    /// </summary>
    /// <typeparam name="T">Класс Сущность самолет</typeparam>
    /// <typeparam name="U">Класс двигателя самолета</typeparam>
    internal class GeneratorAirplane<T, U>
        where T : EntityAirplane
        where U : class, IAirplaneEngines
    {
        private readonly T[] typesOfEntity;
        private readonly U[] typesOfEngines;

        public int NumTypesOfEntity  { get; private set; }
        public int NumTypesOfEngines { get; private set; }

        public GeneratorAirplane(int countTypesOfEntity, int countTypesOfEngines)
        {
            typesOfEntity = new T[countTypesOfEntity];
            typesOfEngines = new U[countTypesOfEngines];
        }
        /// <summary>
        /// Добавляет возможный тип сущности при генерации самолета
        /// </summary>
        /// <param name="type">тип</param>
        /// <returns>Успешно ли проведена операция</returns>
        public virtual bool AddTypeOfEntity(T type)
        {
            if (NumTypesOfEntity >= typesOfEntity.Length)
            {
                return false;
            }
            typesOfEntity[NumTypesOfEntity++] = type;
            return true;
        }
        /// <summary>
        /// Добавляет возможный тип двигателей при генерации самолета
        /// </summary>
        /// <param name="type">тип</param>
        /// <returns>Успешно ли проведена операция</returns>
        public virtual bool AddTypeOfEngines(U type)
        {
            if (NumTypesOfEngines >= typesOfEngines.Length)
            {
                return false;
            }
            typesOfEngines[NumTypesOfEngines++] = type;
            return true;
        }
        /// <summary>
        /// Генерирует объект отрисовки
        /// </summary>
        /// <returns>Возвращает объект отрисовки, либо null, если не были добавлены типы для выборки</returns>
        public DrawningObject? Generate()
        {
            if (NumTypesOfEngines == 0 || NumTypesOfEntity == 0)
            {
                return null;
            }
            var rnd = new Random();
            var airplane = new DrawningAirplane(typesOfEntity[rnd.Next() % NumTypesOfEntity], typesOfEngines[rnd.Next() % NumTypesOfEngines]);
            return new DrawningObject(airplane);
        }
    }
}
